
BASE = 'https://api.tdameritrade.com/v1/'
ACCOUNTS = BASE + 'accounts/'
SEARCH = BASE + 'instruments'
INSTRUMENTS = SEARCH + '/'
QUOTES = BASE + 'marketdata/quotes'
HISTORY = BASE + 'marketdata/%s/pricehistory'
OPTIONCHAIN = BASE + 'marketdata/chains'
MOVERS = BASE + 'marketdata/%s/movers'
